<?php

namespace FiloBlu\Flow\Console\Command;

use Exception;
use FiloBlu\Flow\Model\From\Entity\BulkFactory;
use FiloBlu\Flow\Model\InboundflowFactory;
use Magento\Framework\App\Area;
use Magento\Framework\App\State;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class CopyToConfigurableCommand
 * @package FiloBlu\Flow\Console\Command
 */
class CopyToConfigurableCommand extends Command
{
    /**
     * @var string
     */
    const FLOW_ARGUMENT = 'flow';

    /**
     * @var State
     */
    protected $_state;

    /**
     * @var BulkFactory
     */
    protected $_bulkFactory;

    /**
     * @var InboundflowFactory
     */
    protected $_inboundFlowFactory;

    /**
     * CopyToConfigurableCommand constructor.
     * @param State $state
     * @param BulkFactory $bulkFactory
     * @param InboundflowFactory $inboundFlowFactory
     * @param string $name
     */
    public function __construct(
        State $state,
        BulkFactory $bulkFactory,
        InboundflowFactory $inboundFlowFactory,
        $name = null
    ) {
        $this->_state = $state;
        $this->_bulkFactory = $bulkFactory;
        $this->_inboundFlowFactory = $inboundFlowFactory;
        parent::__construct($name);
    }

    /**
     *
     */
    protected function configure()
    {
        $this->setName('flow:copytoconfigurable')
            ->setDescription('copy attributes from simple to configurable by eavs flow id')
            ->setDefinition([
                new InputArgument(
                    self::FLOW_ARGUMENT,
                    InputArgument::REQUIRED,
                    'Flow'
                ),
            ]);
    }

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int|null|void
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        try {
            $this->_state->setAreaCode(Area::AREA_ADMINHTML);
        } catch (Exception $e) {
            $output->writeln('Areacode was already set, skipping!');
        }

        $flowId = intval($input->getArgument(self::FLOW_ARGUMENT));

        $bulk = $this->_bulkFactory->create();
        $inboundFlow = $this->_inboundFlowFactory->create()->load($flowId);

        if ($inboundFlow === null) {
            $output->writeln("<error>Flow with id {$flowId} does not exists!</error>");
            return;
        }

        if ($inboundFlow->getFlow() !== 'From\Eavs') {
            $output->writeln("<error>Flow with id {$flowId} is not a From\Eavs type!</error>");
            return;
        }

        $bulk->bulkCopyAttributesToConfigurable($inboundFlow);

        $output->writeln('<info>Attributes copied from simples to configurables!</info>');
    }
}
